'use strict';

exports.__esModule = true;
exports['default'] = build;

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError('Cannot call a class as a function'); } }

var _graphqlParser = require('graphql-parser');

var name = 'graphql';

exports.name = name;

function build(babel) {
  var t = babel.types;
  var Transformer = babel.Transformer;

  var GraphQLVisitor = (function () {
    function GraphQLVisitor(refs) {
      _classCallCheck(this, GraphQLVisitor);

      this.refs = refs;
    }

    GraphQLVisitor.prototype.Query = function Query(node) {
      var props = [];
      var query = t.objectExpression(props);

      if (node.fields.length > 0) {
        props.push(compileFields(node.fields));
      }

      return query;
    };

    GraphQLVisitor.prototype.Field = function Field(node) {
      var props = [];

      if (node.alias) {
        props.push(t.property('init', t.identifier('alias'), t.valueToNode(node.alias)));
      }

      if (node.params.length > 0) {
        props.push(t.property('init', t.identifier('params'), t.objectExpression(node.params)));
      }

      if (node.fields.length > 0) {
        props.push(compileFields(node.fields));
      }

      return t.property('init', t.identifier(node.name), t.objectExpression(props));
    };

    GraphQLVisitor.prototype.Argument = function Argument(node) {
      return t.property('init', t.identifier(node.name), node.value);
    };

    GraphQLVisitor.prototype.Literal = function Literal(node) {
      return t.valueToNode(node.value);
    };

    GraphQLVisitor.prototype.Variable = function Variable(node) {
      return t.memberExpression(t.identifier('params'), t.identifier(node.name));
    };

    GraphQLVisitor.prototype.Reference = function Reference(node) {
      return this.refs[node.name];
    };

    return GraphQLVisitor;
  })();

  function compileFields(fields) {
    for (var i = 0; i < fields.length; i++) {
      var field = fields[i];
      if (t.isCallExpression(field)) {
        fields[i] = t.spreadProperty(t.memberExpression(field, t.identifier('fields')));
      }
    }

    return t.property('init', t.identifier('fields'), t.objectExpression(fields));
  }

  function compile(node) {
    var source = '';
    for (var i = 0; i < node.quasis.length; i++) {
      if (i > 0) source += '&' + (i - 1);
      source += node.quasis[i].value.raw;
    }

    return t.functionExpression(null, [t.identifier('params')], t.blockStatement([t.returnStatement((0, _graphqlParser.traverse)((0, _graphqlParser.parse)(source), new GraphQLVisitor(node.expressions)))]));
  }

  return new Transformer(name, {
    TaggedTemplateExpression: {
      enter: function enter(node) {
        if (t.isIdentifier(node.tag, { name: 'graphql' })) {
          return compile(node.quasi);
        }
      }
    }
  });
}